/*
 * interface.c
 *
 * Copyright (c) 2014 Kirill Kryukov
 *
 * This file is a part of the Nucleotide Sequence Clusterizer source code,
 * and may only be used, modified, and distributed under the terms of the
 * license, located in the file LICENSE.txt within this package.
 */

#ifdef INCLUDED_FROM_ALL_C

void help()
{
    fputs("Use -h for help\n",stderr);
}

void print_version()
{
    static char *version_str =
        PROGRAM_NAME ", version " VERSION ", " RELEASE_DATE "\n"
        "Copyright (c) " COPYRIGHT_YEARS " " COPYRIGHT_AUTHORS "\n";
    fputs(version_str,stderr);
}

void print_help()
{
    static char *help_str =
        "Usage: " EXECUTABLE_NAME " [<options>] [<file>...]\n"
        "Options:\n"
        "  -d, --distance D        - Cluster sequences separated by D or fewer substitutions.\n"
        "  -t, --template T        - Use only sequences matching template T.\n"
        "  -v, --version           - Show version.\n"
        "  -h, --help              - Show help.\n";
    fputs(help_str,stderr);
}

/*
 * Converts a zero-terminated string to an unsigned 31-bit integer,
 * taking care of any possible misformatting or overflow.
 * Returns: The number, or 0 on failure.
 */
unsigned parse_uint31(char *str,char *name,int talk,int quit)
{
    char *start = str;
    char *c;
    unsigned total = 0;
    static const char cant_parse[] = "Can't parse %s (\"%s\")\n";

    assert(str);
    assert(name);

    while (isspace(*(unsigned char*)start)) { start++; }
    if (*start == '+') { start++; }
    while (*start == '0') { start++; }

    if (!*start)
    {
        if (start != str && *(start-1) == '0') { return 0; }
        if (talk) { fprintf(stderr,cant_parse,name,str); }
        if (quit) { exit(1); } else { return 0; }
    }

    for (c = start; *c; c++)
    {
        if (!isdigit(*(unsigned char*)c))
        {
            if (talk) { fprintf(stderr,cant_parse,name,str); }
            if (quit) { exit(1); } else { return 0; }
        }
    }

    if (c <= start)
    {
        if (talk) { fprintf(stderr,cant_parse,name,str); }
        if (quit) { exit(1); } else { return 0; }
    }

    if (strlen(start) > 10 || (strlen(start) == 10 && strcmp(start,"2147483647") > 0))
    {
        if (talk) { fprintf(stderr,cant_parse,name,str); }
        if (quit) { exit(1); } else { return 0; }
    }

    for (c = start; *c; c++)
    {
        total = total * 10 + (*c - '0');
    }

    return total;
}

void parse_options(int argc,char **argv)
{
    enum opt_codes { oc_distance = 256, oc_template };
    static struct option long_options[] = {
        {"version" , no_argument, &opt_version     , 1},
        {"help"    , no_argument, &opt_help        , 1},
        {"distance", required_argument, 0, oc_distance},
        {"template", required_argument, 0, oc_template},
        {0, 0, 0, 0} };
    int c;
    opterr = 0;
    do {
        int curind = optind;
        c = getopt_long(argc,argv,"vhcgji:o:d:n:m:e:x:t:L:",long_options,0);
        switch (c)
        {
            case 'v': opt_version = 1; break;
            case 'h': opt_help = 1; break;
            case 'd': case oc_distance: dist = parse_uint31(dist_str = optarg,"distance",1,1); break;
            case 't': case oc_template: template_parameter = optarg; break;
            case '?':
                if (optopt) { fprintf(stderr,"Unrecognized option -%c\n",optopt); }
                else { fprintf(stderr,"Unrecognized option \"%s\"\n",argv[curind]); }
                help();
                exit(1);
                break;
            case 0: break;
            case -1: break;
            default: assert(0);
        }
    }
    while (c != -1);

    if (optind > 0 && optind < argc) { argv_index_of_first_path = optind; }
}

void read_command_line(int argc,char **argv)
{
    parse_options(argc,argv);
    if (opt_version) { print_version(); }
    if (opt_help) { print_help(); }

    if (!dist_str)
    {
        if (!opt_version && !opt_help) { fprintf(stderr,"Distance is not specified (\"--distance\" option)\n"); help(); }
        exit(1);
    }
    if (dist > 3)
    {
        fprintf(stderr,"Too large distance. Only distances up to 3 are supported.\n");
        exit(1);
    }

    if (!template_parameter || !*template_parameter)
    {
        if (!opt_version && !opt_help) { fprintf(stderr,"Template is not specified (\"--template\" option)\n"); help(); }
        exit(1);
    }
}

#endif
