/*
 * read.c
 *
 * Copyright (c) 2014 Kirill Kryukov
 *
 * This file is a part of the Nucleotide Sequence Clusterizer source code,
 * and may only be used, modified, and distributed under the terms of the
 * license, located in the file LICENSE.txt within this package.
 */

#ifdef INCLUDED_FROM_ALL_C

/*
 * Stores one barcode in binary form in the current block of barcodes.
 */
void store_barcode(unit_t *u)
{
    assert(u);

    if (!last_block || n_barcodes_in_last_block >= barcodes_per_block)
    {
        block_t *new_block = (block_t*)malloc1(sizeof(block_t));
        if (last_block) { last_block->next = new_block; }
        else { first_block = new_block; }
        last_block = new_block;
        n_barcodes_in_last_block = 0;
        n_units_used_in_last_block = 0;
        n_blocks++;
    }

    memcpy(&last_block->units[n_units_used_in_last_block],u,bytes_per_barcode);
    n_barcodes_in_last_block++;
    n_barcodes_total++;
    n_units_used_in_last_block += units_per_barcode;
}

/*
 * Reads a barcode from stream and converts it to binary form.
 * Returns:
 *    0 - barcode was read successfully.
 *    1 - barcode was skipped due to not matching template.
 *    2 - barcode was skipped due to parsing error.
 *    3 - end of file.
 */
int read_one_barcode(file_t *file)
{
    unit_t u[units_per_barcode];
    int u_index = 0;
    unsigned u_filled = 0;
    unsigned u_filled_total = 0;
    int t_pos = 0;

    assert(file);

    u[0] = 0ull;

    /*printf("Reading barcode\n");*/

    skip_line_breaks(file);

    /*printf("    skipped line breaks\n");*/

    if (!read_file(file))
    {
        /*printf("    end of file!\n");*/
        return 3;
    }

    if (*file->buf_start != '>')
    {
        /*printf("    The next character is '%c' - expecting '>'! => skipping to the next sequence\n",*file->buf_start);*/
        skip_until_next_fasta_sequence(file);
        return 2;
    }

    file->buf_start++;
    skip_until_line_break(file);
    skip_line_breaks(file);

    /*printf("    skipped name\n");*/

    while (read_file(file))
    {
        unsigned char c;
        unsigned isnuc;
        while (file->buf_start < file->buf_end && (isnuc = is_nucleotide[c = *(unsigned char*)file->buf_start]))
        {
            /*printf("    position %d, template: '%c', reading '%c'\n",t_pos,template[t_pos],*file->buf_start);*/

            if (use_barcode_position[t_pos])
            {
                if (u_filled_total >= barcode_complexity) { skip_until_next_fasta_sequence(file); return 1; }
                if (u_filled >= BP_PER_UNIT) { u_index++; u[u_index] = 0ull; u_filled = 0; }
                u[u_index] = (u[u_index] << 2) | (isnuc-1);
                u_filled++;
                u_filled_total++;
            }
            else
            {
                if (toupper(c) != template[t_pos]) { skip_until_next_fasta_sequence(file); return 1; }
            }
            t_pos++;
            file->buf_start++;
        }
        if (file->buf_start < file->buf_end) { break; }
    }
    if (*file->buf_start != 13 && *file->buf_start != 10) { skip_until_next_fasta_sequence(file); return 1; }
    if (u_filled_total == barcode_complexity) { u[u_index] <<= (BP_PER_UNIT - u_filled)*2; store_barcode(u); }
    return 0;
}

void load_barcodes_from_file(char* path)
{
    file_t file = {0,0,0,0,0};
    int error;
    long long n_parse_errors = 0;
    long long n_template_mismatches = 0;
    long long n_loaded = 0;

    if (path)
    {
        file_t *f = open_file(&file,path,"rb",BUFSIZE);
        if (!f) { fprintf(stderr,"Error: Can't open \"%s\"\n",path); return; }
        fprintf(stderr,"Scanning \"%s\"\n",path);
    }
    else
    {
        make_buffered_file_from_stream(&file,stdin,BUFSIZE);
        fprintf(stderr,"Scanning standard input\n");
    }

    while ( (error = read_one_barcode(&file)) < 3)
    {
        if (error)
        {
            if (error == 1) { n_template_mismatches++; }
            else { n_parse_errors++; }
        }
        else { n_loaded++; }
    }

    if (n_parse_errors) { fprintf(stderr," - %lld parse errors\n",n_parse_errors); }
    if (n_template_mismatches) { fprintf(stderr," - %lld template mismatches\n",n_template_mismatches); }
    fprintf(stderr," - Loaded %lld sequences matching template\n",n_loaded);

    close_file(&file);
    free_file_buffer(&file);
}

#endif
